---
id: docker-images
title: Running in Docker
description: Example Docker images to run your crawlers
---

import ApiLink from '@site/src/components/ApiLink';

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';
import CodeBlock from '@theme/CodeBlock';

import jsNodeDocker from '!!raw-loader!./docker_node_js.txt';
import tsNodeDocker from '!!raw-loader!./docker_node_ts.txt';
import jsBrowserDocker from '!!raw-loader!./docker_browser_js.txt';
import tsBrowserDocker from '!!raw-loader!./docker_browser_ts.txt';

Running headless browsers in Docker requires a lot of setup to do it right. But there's no need to worry about that, because we already created base images that you can freely use. We use them every day on the [Apify Platform](./apify-platform).

All images can be found in their [GitHub repo](https://github.com/apify/apify-actor-docker) and in our [DockerHub](https://hub.docker.com/orgs/apify).

## Overview

Browsers are pretty big, so we try to provide a wide variety of images to suit the specific needs. Here's a full list of our Docker images.

- [`apify/actor-node`](#actor-node)
- [`apify/actor-node-puppeteer-chrome`](#actor-node-puppeteer-chrome)
- [`apify/actor-node-playwright`](#actor-node-playwright)
- [`apify/actor-node-playwright-chrome`](#actor-node-playwright-chrome)
- [`apify/actor-node-playwright-firefox`](#actor-node-playwright-firefox)
- [`apify/actor-node-playwright-webkit`](#actor-node-playwright-webkit)

## Versioning

Each image is tagged with up to 2 version tags, depending on the type of the image. One for Node.js version and second for pre-installed web automation library version. If you use the image name without a version tag, you'll always get the latest available version.

> We recommend always using at least the Node.js version tag in production Dockerfiles. It will ensure that a future update of Node.js will not break our automations.

### Node.js versioning

Our images are built with multiple Node.js versions to ensure backwards compatibility. Currently, Node.js **versions 16 and 18 are supported** (legacy versions still exist, see DockerHub). To select the preferred version, use the appropriate number as the image tag.

```dockerfile
# Use Node.js 16
FROM apify/actor-node:16
```

### Automation library versioning

Images that include a pre-installed automation library, which means all images that include `puppeteer` or `playwright` in their name, are also tagged with the pre-installed version of the library. For example, `apify/actor-node-puppeteer-chrome:16-13.7.0` comes with Node.js 16 and Puppeteer v13.7.0. If you try to install a different version of Puppeteer into this image, you may run into compatibility issues, because the Chromium version bundled with `puppeteer` will not match the version of Chromium that's pre-installed.

Similarly `apify/actor-node-playwright-firefox:14-1.21.1` runs on Node.js 14 and is pre-installed with the Firefox version that comes with v1.21.1.

Installing `apify/actor-node-puppeteer-chrome` (without a tag) will install the latest available version of Node.js and `puppeteer`.

### Pre-release tags

We also build pre-release versions of the images to test the changes we make. Those are typically denoted by a `beta` suffix, but it can vary depending on our needs. If you need to try a pre-release version, you can do it like this:

```dockerfile
# Without library version.
FROM apify/actor-node:16-beta
```

```dockerfile
# With library version.
FROM apify/actor-node-playwright-chrome:16-1.10.0-beta
```

## Best practices

- Node.js version tag should **always** be used.
- The automation library version tag should be used for **added security**.
- Asterisk `*` should be used as the automation library version in our `package.json` files.

It makes sure the pre-installed version of Puppeteer or Playwright is not re-installed on build. This is important, because those libraries are only guaranteed to work with specific versions of browsers, and those browsers come pre-installed in the image.

```dockerfile
FROM apify/actor-node-playwright-chrome:16
```

```json
{
    "dependencies": {
        "crawlee": "^3.0.0",
        "playwright": "*"
    }
}
```

### Warning about image size

Browsers are huge. If you don't need them all in your image, it's better to use a smaller image with only the one browser you need.

You should also be careful when installing new dependencies. Nothing prevents you from installing Playwright into the`actor-node-puppeteer-chrome` image, but the resulting image will be about 3 times larger and extremely slow to download and build.

When you use only what you need, you'll be rewarded with reasonable build and start times.

## Apify Docker Images

### actor-node

This is the smallest image we have based on Alpine Linux. It does not include any browsers, and it's therefore best used with <ApiLink to="cheerio-crawler/class/CheerioCrawler">`CheerioCrawler`</ApiLink>. It benefits from lightning fast builds and container startups.

&#8203;<ApiLink to="puppeteer-crawler/class/PuppeteerCrawler">`PuppeteerCrawler`</ApiLink>, <ApiLink to="playwright-crawler/class/PlaywrightCrawler">`PlaywrightCrawler`</ApiLink> and other browser based features will **NOT** work with this image.

```dockerfile
FROM apify/actor-node:16
```

### actor-node-puppeteer-chrome

This image includes Puppeteer (Chromium) and the Chrome browser. It can be used with <ApiLink to="cheerio-crawler/class/CheerioCrawler">`CheerioCrawler`</ApiLink> and <ApiLink to="puppeteer-crawler/class/PuppeteerCrawler">`PuppeteerCrawler`</ApiLink>, but **NOT** with <ApiLink to="playwright-crawler/class/PlaywrightCrawler">`PlaywrightCrawler`</ApiLink>.

The image supports XVFB by default, so you can run both `headless` and `headful` browsers with it.

```dockerfile
FROM apify/actor-node-puppeteer-chrome:16
```

### actor-node-playwright

A very large and slow image that can run all Playwright browsers: Chromium, Chrome, Firefox, WebKit. Everything is installed. If you need to develop or test with multiple browsers, this is the image to choose, but in most cases, it's better to use the specialized images below.

```dockerfile
FROM apify/actor-node-playwright:16
```

### actor-node-playwright-chrome

Similar to [`actor-node-puppeteer-chrome`](#actor-node-puppeteer-chrome), but for Playwright. You can run <ApiLink to="cheerio-crawler/class/CheerioCrawler">`CheerioCrawler`</ApiLink> and <ApiLink to="playwright-crawler/class/PlaywrightCrawler">`PlaywrightCrawler`</ApiLink>, but **NOT** <ApiLink to="puppeteer-crawler/class/PuppeteerCrawler">`PuppeteerCrawler`</ApiLink>.

It uses the [`PLAYWRIGHT_SKIP_BROWSER_DOWNLOAD`](https://playwright.dev/docs/api/environment-variables/) environment variable to block installation of more browsers into the image to keep it small. If you want more browsers, either use the [`actor-node-playwright`](#actor-node-playwright) image override this env var.

The image supports XVFB by default, so we can run both `headless` and `headful` browsers with it.

```dockerfile
FROM apify/actor-node-playwright-chrome:16
```

### actor-node-playwright-firefox

Same idea as [`actor-node-playwright-chrome`](#actor-node-playwright-chrome), but with Firefox
pre-installed.

```dockerfile
FROM apify/actor-node-playwright-firefox:16
```

### actor-node-playwright-webkit

Same idea as [`actor-node-playwright-chrome`](#actor-node-playwright-chrome), but with WebKit
pre-installed.

```dockerfile
FROM apify/actor-node-playwright-webkit:16
```

## Example Dockerfile

To use the above images, it's necessary to have a [`Dockerfile`](https://docs.docker.com/engine/reference/builder/). You can either use this example, or bootstrap your projects with the [Crawlee CLI](../introduction/setting-up) which automatically adds the correct Dockerfile into our project folder.

<Tabs>
    <TabItem value="node+js" label="Node+JavaScript">
        <CodeBlock language="dockerfile">
            {jsNodeDocker}
        </CodeBlock>
    </TabItem>
    <TabItem value="node+ts" label="Node+TypeScript">
        <CodeBlock language="dockerfile">
            {tsNodeDocker}
        </CodeBlock>
    </TabItem>
    <TabItem value="browser+js" label="Browser+JavaScript">
        This example is for Playwright. If you want to use Puppeteer, simply replace <b>playwright</b>
        with <b>puppeteer</b> in the <code>FROM</code> declaration.
        <CodeBlock language="dockerfile">
            {jsBrowserDocker}
        </CodeBlock>
    </TabItem>
    <TabItem value="browser+ts" label="Browser+TypeScript">
        This example is for Playwright. If you want to use Puppeteer, simply replace <b>playwright</b>
        with <b>puppeteer</b> in both <code>FROM</code> declarations.
        <CodeBlock language="dockerfile">
            {tsBrowserDocker}
        </CodeBlock>
    </TabItem>
</Tabs>
